/*

    MiniM

    ZDLL function interfaces

    Copyright (C) Eugene Karataev
    All rights reserved

*/

#ifndef __ZDLL_H__
#define __ZDLL_H__

#ifdef __cplusplus
extern "C" {
#endif

/* max string size */
#define MINIM_STR_MAX ( 32 * 1024 )

#pragma pack(push,1)
typedef struct _MINIM_STR
{
  union{
    unsigned short len;
    unsigned short type;
  };
  unsigned char data[ MINIM_STR_MAX];
} MINIM_STR;
#pragma pack(pop)

/* non-string types */
#define MT_INT32      ( 0x0000FFFF )
#define MT_INT64      ( 0x0000FFFE )
#define MT_DOUBLE     ( 0x0000FFFD )

/* callback functions */
typedef void ( __stdcall * errstr_t)( MINIM_STR* errstr);
typedef int ( __stdcall * execute_t)( MINIM_STR* str);
typedef int ( __stdcall * eval_t)( MINIM_STR* expr, MINIM_STR* res);
typedef double ( __stdcall * getdouble_t)( MINIM_STR* val);
typedef long ( __stdcall * getint32_t)( MINIM_STR* val);
typedef __int64 ( __stdcall * getint64_t)( MINIM_STR* val);
typedef void ( __stdcall * getstr_t)( MINIM_STR* val, MINIM_STR* str);
typedef void ( __stdcall * setdouble_t)( double val, MINIM_STR* str);
typedef void ( __stdcall * setint32_t)( long val, MINIM_STR* str);
typedef void ( __stdcall * setint64_t)( __int64 val, MINIM_STR* str);

/* return error types */

/* all done ok */
#define ZDLL_CALLBACK_DONE       ( 0 )
/* syntax errors in parameters */
#define ZDLL_CALLBACK_SYNTAX     ( 1 )
/* some parameters null */
#define ZDLL_CALLBACK_PARAMETERS ( 2 )
/* argc is more than supported */
#define ZDLL_CALLBACK_ARGC       ( 3 )
/* undefined variable */
#define ZDLL_CALLBACK_UNDEFINED  ( 4 )
/* database or process error */
#define ZDLL_CALLBACK_ERROR      ( 5 )
/* halt command was in minimono context */
#define ZDLL_CALLBACK_HALT       ( 6 )

typedef int (__stdcall * userfunc_t)( const char* name, int argc,
  MINIM_STR** argv, MINIM_STR* result);
typedef int (__stdcall * userdo_t)( const char* name, int argc,
  MINIM_STR** argv);
typedef int (_stdcall * readlocal_t)( const char* name, int argc,
  MINIM_STR** argv, MINIM_STR* result);
typedef int (_stdcall * writelocal_t)( const char* name, int argc,
  MINIM_STR** argv, MINIM_STR* value);
typedef int (_stdcall * readglobal_t)( const char* name, const char* database,
  int argc, MINIM_STR** argv, MINIM_STR* result);
typedef int (_stdcall * writeglobal_t)( const char* name, const char* database,
  int argc, MINIM_STR** argv, MINIM_STR* value);
typedef int (__stdcall * kill_local_t)( const char* name, int argc,
  MINIM_STR** argv);
typedef int (__stdcall * kill_global_t)( const char* name,
  const char* database, int argc, MINIM_STR** argv);
typedef int (__stdcall * set_test_t)( int svn_test_value);
typedef int (__stdcall * order_local_t)( const char* name, int argc,
  MINIM_STR** argv, int forward, MINIM_STR* result);
typedef int (__stdcall * order_global_t)( const char* name,
  const char* database, int argc, MINIM_STR** argv, int forward,
  MINIM_STR* result);
typedef int (__stdcall * inc_local_t)( const char* name, int argc,
  MINIM_STR** argv, MINIM_STR* result);
typedef int (__stdcall * inc_global_t)( const char* name,
  const char* database, int argc,
  MINIM_STR** argv, MINIM_STR* result);
typedef int (__stdcall * data_local_t)( const char* name, int argc,
  MINIM_STR** argv, MINIM_STR* result);
typedef int (__stdcall * data_global_t)( const char* name,
  const char* database, int argc,
  MINIM_STR** argv, MINIM_STR* result);

/* callback function list */
#pragma pack(push,1)
typedef struct _ZDLLCB
{
  errstr_t        ErrStr;
  execute_t       Execute;
  eval_t          Eval;
  getdouble_t     GetDouble;
  getint32_t      GetInt32;
  getint64_t      GetInt64;
  getstr_t        GetStr;
  setdouble_t     SetDouble;
  setint32_t      SetInt32;
  setint64_t      SetInt64;
  userfunc_t      UserFunc;
  userdo_t        UserDo;
  readlocal_t     ReadLocal;
  writelocal_t    WriteLocal;
  readglobal_t    ReadGlobal;
  writeglobal_t   WriteGlobal;
  kill_local_t    KillLocal;
  kill_global_t   KillGlobal;
  set_test_t      SetTEST;
  order_local_t   OrderLocal;
  order_global_t  OrderGlobal;
  inc_local_t     IncLocal;
  inc_global_t    IncGlobal;
  data_local_t    DataLocal;
  data_global_t   DataGlobal;
} ZDLLCB;
#pragma pack(pop)

/* ZDLL function */
/* each function accept variable-lenght arguments list */
typedef int ( __stdcall * zdllfunc_t)( ZDLLCB* cbfunc, MINIM_STR* result,
  int argc, MINIM_STR** argv);

/* one ZDLL function definition */
#pragma pack(push,1)
typedef struct _ZDLLFUNC
{
  zdllfunc_t ZDLLFunc;
  char*      FuncName;
} ZDLLFUNC;
#pragma pack(pop)

/* ZDLL exported function */
typedef ZDLLFUNC* ( __stdcall * zdllfuncexport_t)( void);

/*

    ZDLL module should export function with name "ZDLL" as zdllfuncexport_t.
    This function should return pointer to array of structures ZDLLFUNC,
    with zeroed last.

    See for comments ZDLL examples in minim zdll subdirectory.

*/

#ifdef __cplusplus
}
#endif

/* end of __ZDLL_H__ */
#endif